'===============================================================================
'        <Program Beginning For "Multi-Z CPU Communications" in Slot 2 / J2 >
'===============================================================================
'
'========================================================================================
'Module Title: Multi-Z CPU Comm 2 R1.Bas - "Main Subroutine"
'========================================================================================
'Author: Zbasicandy
'========================================================================================
'Date:
'========================================================================================
'Operating System: XP
'========================================================================================
'ZBasic IDE Version: 1.1.1 / ZBasic Compiler version 1.29 / ZX-40 Firmware 1.2 / ZLoad v1.1.4
'========================================================================================
'Target: ZX-40a with 256k memory 
'========================================================================================
'Hardware: "Multi-Z CPU with BackPlane" 
'Support (Driver) Modules Needed:
'
'           ComSupport_R4.bas - for serial commuications - very useful communication routines 
'           Z40_Pin_I_O_R2.bas - for generic / ZX-40 Pin Input/Output functions/routines          
'
'========================================================================================
'Comports used: Software Comports 3,4 on ZX-40x for "Multi-Z CPU" hardware. 
'========================================================================================
'Bugs fixes and updates:
'
'
'
'
'
'
'****Warning**** This code is subject to change due to enhancements and bug fixes. 
'Treat this code as prototype code!
'========================================================================================
'
'===============
'System Options:
'===============

Option Explicit            'default (OFF) - implicit variable definition not used
Option Language Zbasic     'default
Option Base 1              'arrays start with 1 not 0
Option TargetCPU ZX40      'ZX40 (ZX-40 Atmega32); ZX40a (ZX-40a ATmega644)
Option CodeLimit 32K       '(1024 blocks)error if exceeded
Option SignOn On           'show sign-on on reset or boot
Option StringSize 40       '40 chars (default is 20) 'matches buffers
Option Strict Off          'zbasic default
Option AllocStr On         'zbasic default
Option Strict Off          'default
'
'============================================================================
'Pin State Meaning:
'zxInputTriState The pin should be a tri-state input (pull-up disabled).
'zxInputPullUp   The pin should be an input with the pull-up enabled.
'zxOutputLow     The pin should be an output set to logic zero.
'zxOutputHigh    The pin should be an output set to logic one.
'Warning - Keep all comports Tx(s) high and Rx(s) pulled high in software (idle)
'This prevents char ejects on start-up
'============================================================================
'                         I/O options - For ZX-40x ONLY!
'============================================================================
'Note: All pins are initilize as an input on power up.
'
Option Pin1  zxOutputLow        'PB0,  J2 pin 22, JR2 pin 1
Option Pin2  zxOutputLow        'PB1,  J2 pin 10, JL2 pin 2
Option Pin3  zxOutputLow        'INT2, J2 pin 23, JL2 pin 1
Option Pin4  zxOutputLow        'PB3,  J2 pin 11, JR1 pin 2

'-------------------------------------------------------------------------
'Do not change or enable!
'------------------------
'SPI / EEProm
'Option Pin5  zxInputtristate   'SS CS for SPI - for ref only
'Option Pin6  zxInputtristate   'MOSI SPI - for ref only
'Option Pin7  zxInputtristate   'MISO SPI - for ref only
'Option Pin8  zxInputtristate   'SCK SPI - for ref only
'Com1 program port
'Option Pin14 zxInputPullup     'Com1 Rx - Download port - for ref only
'Option Pin15 zxOutputHigh      'Com1 Tx - Download port - for ref only
'-------------------------------------------------------------------------
'
Option Pin16 zxInputPullUp      'INT0, J3 pin 3 (Multi-processing connector J3)
Option Pin17 zxInputPullUp      'INT1, J3 pin 4 (Multi-processing connector J3)
Option Pin18 zxOutputLow        'PD4,  Run LED -> High = On & Low = Off
Option Pin19 zxOutputHigh       'PD5,  J3 pin 1 (Multi-processing connector J3) Pulled-up on board
Option Pin20 zxOutputHigh       'PD6,  J3 pin 2 (Multi-processing connector J3) Pulled-up on board

Option Pin21 zxOutputLow        'PD7,      J2 pin 9,  JR2 pin 2 
Option Pin22 zxOutputLow        'SCL I2C,  J2 pin 21, JL3 pin 1
Option Pin23 zxOutputLow        'SDA I2C,  J2 pin  8, JL3 pin 2 
Option Pin24 zxInputPullUp      'Rx3,Com3  J2 pin  7, JR3 pin 2 <--RS232-converted on CPU x Board
Option Pin25 zxOutputHigh       'Tx3,Com3  J2 pin 20, JR3 pin 1 <--RS232-converted on CPU x Board
Option Pin26 zxOutputLow        'PC4,      J2 pin 19, JL4 pin 1
Option Pin27 zxOutputLow        'PC5,      J2 pin  6, JL4 pin 2
Option Pin28 zxOutputLow        'PC6,      J2 pin 18, JR4 pin 1
Option Pin29 zxOutputLow        'PC7,      J2 pin  5, JR4 pin 2

Option Pin33 zxOutputLow        'PA7,      J2 pin 17, JL5 pin 1 
Option Pin34 zxOutputLow        'PA6,      J2 pin  6, JL5 pin 2
Option Pin35 zxOutputLow        'PA5,      J2 pin 16, JR5 pin 1
Option Pin36 zxOutputLow        'PA4,      J2 pin  3, JR5 pin 2
Option Pin37 zxOutputLow        'PA3,      J2 pin 15, JL6 pin 1
Option Pin38 zxOutputLow        'PA2,      J2 pin  2, JL6 pin 2
Option Pin39 zxOutputLow        'PA1,      J2 pin 14, JR6 pin 1   
Option Pin40 zxOutputLow        'PA0,      Interrupt LED - On CPU Board -> High = On & Low = Off
'========================================================================================
'                            End of I/O Options for ZX-40x
'========================================================================================

'------------------------
'Multi-Z Multi-Processing 
'------------------------
'
'-------------------------------------------------------
' Multi-Z CPU Multi-Processing I/O Selection Table Setup 
' For reference only.
'------------------------------------------------------------------------
' Card / Slot Position                       | J1   |  J2 Comm   | J3    |
'     CPU Selection -->                      |      |     X      |       |
'                                            |Slave1|   Master   |Slave2 |
'-----------------------------------------------------------------------
' Multi-Proc. Con. J3 Pin 1  ZX-40x pin 19   | Tx4  | Out / Tx4  | Tx4   |
'                  J3 Pin 2             20   | Out  | Out / Tx5  | Out   |
'                  J3 Pin 3             16   | INT0 | Rx4 / INT0 | INT0  |
'                  J3 Pin 4             17   | Rx4  | Rx5 / INT1 | Rx4   |
'------------------------------------------------------------------------
'
'-------------------------------------
' J3 Cable Wiring for Multi-Processing
'-------------------------------------
'Note: Pin 1 is the closest to the top edge of the CPU board.
'Note: Quantity of 6 1.0 K series current limiting resistor(s) are inserted between all 
'wiring connections for protection. The resistors are soldered and heat shrink in the cabling.
'Note: Check schematic for the Multi-Z J3 wiring cabling diagram between Multi-Z CPUs.
'
'Pin 1 of J1 goes to Pin 3 of J2 - for Multi-Z CPU slot 1 or J1
'Pin 3 of J1 goes to Pin 1 of J2 - for Multi-Z CPU slot 1 or J1
'Pin 1 of J3 goes to Pin 3 of J2 - for Multi-Z CPU slot 3 or J3
'Pin 3 of J3 goes to Pin 2 of J2 - for Multi-Z CPU slot 3 or J3
'--------------------------------------------------------------


'=========================================================================================
'Program Definitions:
'=========================================================================================
'
'
'============================================================================
'Constants:
'[Public | Private] Const <name> As <type> = <value> (default private)
'============================================================================

'-----------------------------
' Port Constants for all ports
'-----------------------------
'Do not modify!
Private Const Port3 as byte = 3           
Private Const Port4 as byte = 4
Private Const Port5 as byte = 5
'Private Const Port6 as byte = 6
'-----------------------------

'General Constants
Private Const Run_LED as byte = 18
Private Const Int_LED as byte = 40
Private Const Multi_Z_Slot_1_INT0 as byte = 19 
Private Const Multi_Z_Slot_3_INT0 as byte = 20  
Private Const On as  byte =  1
Private Const Off as byte =  0

'============================================================================
'Variables:
'{Public | Private | Dim} <name> As <type> (default private)
'============================================================================

	

'============================================================================
'Subroutine declarations:
'============================================================================

'-------------------------------------------------------------------------------------------
Private Sub Get_Slave1_Data(Byref Receive_OK as Boolean) 'Get data from CPU 1 in slot 1 / J1
Dim Num_Char as Integer
Dim Rx_Str as String
Dim TimeOutS as Single
Dim Success as Boolean

ClearInputBuffer(Port4)
Call PutPin(Multi_Z_Slot_1_INT0,Off) 'High level(On = 0) for INTO for Multi-Z CPU in slot 1 / J1  
Call Delay(0.015) ' 9600/10 = 960  1/960 = ~1ms  10 bytes~10ms
Call PutPin(Multi_Z_Slot_1_INT0,ON)  'High level(Off = 1) for INTO for Multi-Z CPU in slot 1 / J1 
Call PutPin(Int_LED,On) 'Turn on interrupt LED to indicate receive data
Rx_str = ""
TimeOutS = 1.0 ' 1 sec timeout max.
Num_Char = 10  ' 10 char bytes to receive
Call Get_String_40(Port4,Rx_Str,Num_Char,TimeOutS, Success) '40 chars max
Debug.print "Rx_Str = "; Rx_Str
Debug.print "Num char = "; Cstr(Num_char)
If Success then
Debug.print "Slave 1 Receive Data Success"
Receive_OK = True
Else
Debug.print "Slave 1 Receive Data Failure"
Receive_OK = False
End if
Call PutPin(Int_LED,Off) 'Turn off interrupt LED
End Sub	
'-------------------------------------------------------------------------------------------
Private Sub Get_Slave3_Data(Byref Receive_OK as boolean) 'Get data from CPU 3 in slot 3 / J3
Dim Num_Char as Integer
Dim Rx_Str as String
Dim TimeOutS as Single
Dim Success as Boolean

ClearInputBuffer(Port5)
Call PutPin(Multi_Z_Slot_3_INT0,Off) 'High level(On = 0) for INTO for Multi-Z CPU in slot 3 / J3  
Call Delay(0.015) ' 9600/10 = 960  1/960 = ~1ms  10 bytes~10ms
Call PutPin(Multi_Z_Slot_3_INT0,ON)  'High level(Off = 1) for INTO for Multi-Z CPU in slot 3 / J3 
Call PutPin(Int_LED,On) 'Turn on interrupt LED to indicate receive data
Rx_str = ""
TimeOutS = 1.0 ' 1 sec timeout max.
Num_Char = 10  ' 10 char bytes to receive
Call Get_String_40(Port5,Rx_Str,Num_Char,TimeOutS, Success) '40 chars max
Debug.print "Rx_Str = "; Rx_Str
Debug.print "Num char = "; Cstr(Num_char)
If Success then
Debug.print "Slave 3 Receive Data Success"
Receive_OK = True
Else
Debug.print "Slave 3 Receive Data Failure"
Receive_OK = False
End if
Call PutPin(Int_LED,Off) 'Turn off interrupt LED
End Sub	

'-------------------------------------------------------------------------------------------
Private Sub Com3_Turn_Around_Test() 
'Install jumper on RS232 connector on pins 2-3 on the backplane
Dim OK as Boolean 'For com3 turnaround test
Call Comport_TurnaroundTest(Port3, OK )
If OK then
	Debug.Print "Port 3 is OK"
	Else
	Debug.Print "Port 3 is Not OK!"
End if
End Sub
'-------------------------------------------------------------------------------------------
Private Sub Cycle_LEDS() 'Cycle all LEDs on Power-up or Reset - For troubleshooting
Dim i as Integer
For i = 0 to 5 ' 6 times
Call PutPin(Run_LED,On)
Call PutPin(Int_LED,On)
Call Delay(0.2)
Call PutPin(Run_LED,Off)
Call PutPin(Int_LED,Off)
Call Delay(0.2)
Next
End Sub
'============================================================================	 
'
'============================================================================
'Function declarations:
'============================================================================

'============================================================================
'Tasks Init. Setup
'============================================================================
'Dim TaskStack1(1 to 110) as Byte  '
'Dim TaskStack2(1 to 50) as Byte  ' 
'Dim TaskStack3(1 to 50) as Byte ' spare
'============================================================================

'--------------------------
'Main Subroutine
'--------------------------
Sub Main()
Dim Receive_OK as Boolean

'------------------------------------------------------------------------------------------
'                       Serial Port(s) Communications Setup
'------------------------------------------------------------------------------------------

'----------------------------------------------------
'Set number of Communications channels and max baud rate
'----------------------------------------------------
Call ComChannels(3, 9600) ' Max 9600 using 4 comports
'----------------------------------------------------
'Define "Multi-Z CPU 1" ZX-40x Commuications Serial I/O
'---------------------------------------------------

Call DefineCom(3, 24, 25, &H08) ' Com 3 ,rec pin 24 tran pin 25 - 8,N,1 <--- External RS232 communications port 
Call DefineCom(4, 16, 19, &H08) ' Com 4 ,rec pin 16 tran pin 19 - 8,N,1 <--- For Multi-Processing 
Call DefineCom(5, 17, 20, &H08) ' Com 5 ,rec pin 17 tran pin 20 - 8,N,1 <--- For Multi-Processing  
'Call DefineCom(6, x, x, &H08) ' Com 6 ,rec pin  x tran pin x - 8,N,1 

'------------------------------
'Software Serial ports "opened"
'------------------------------
Call OpenSerialPort(3,9600) 'Com 3 <--RS232-converted on CPU Board (JR3 pins 1 & 2 - Tx & Rx) 
Call OpenSerialPort(4,9600) 'Com 4 <-- For TTL Multi-Z CPU Processing - Do Not Use! 
Call OpenSerialPort(5,9600) 'Com 5 <-- For TTL Multi-Z CPU Processing - Do Not Use! 
'Call OpenSerialPort(6,9600) 'Com 6 

'-----------------------------------------------------------------------------------------
'                          End Of Communications Setup
'-----------------------------------------------------------------------------------------




'===============================
'Call Task(s) for Multi-Tasking.
'===============================
'CallTask "CPU_1_INT0", TaskStack1   'Check for Multi-Tasking - make sure cabling is installed!
'CallTask "CPU_3_INT1", TaskStack2  'Check for Multi-Tasking - make sure cabling is installed!
   'CallTask " ", TaskStack2 'spare
'-------------------------------
Do
Call Get_Slave1_Data(Receive_OK)
'Call Get_Slave3_Data(Receive_OK)
CALL TogglePin(Run_LED) 'Toggle the red RUN LED - Multi-Z CPU Running indication.
Call Delay(0.01)
Loop

End Sub


'===================================================================================
'                         End of "Main" Application Subroutine
'===================================================================================




'===================================================================================
'                           CPU 1 Interrupt 0 TaskStack1  
'===================================================================================
Sub CPU_1_INT0() 'From CPU 1 in slot 1 / J1 on INT0
Dim x as byte
Dim Success as Boolean
Dim Num_Char as Integer
Dim Rx_Str as String
Dim TimeOutS as Single

	Do
	'Warning - Do not use this routine if the cabling between Multi-Z CPUs is not connected!
	'Call WaitForInterrupt(zxPinLow,0) 'Wait for low level from Multi-Z CPU in slot 1 / J1
	'Wait 10-15 ms for data in buffer (10 bytes)
	
	'Call PutPin(Multi_Z_Slot_1_INT0,ON)  'High level (Off) for INTO for Multi-Z CPU in slot 1 / J1 	
	'Call LockTask()
	'ClearInputBuffer(Port4)
	'Rx_str = ""
	'TimeOutS = 1.0 ' 1 sec
	'Num_Char = 10
	'Call Get_String_40(Port4,Rx_Str,Num_Char,TimeOutS, Success) '40 chars max
	'Debug.print "Rx_Str = "; Rx_Str
	'Debug.print "Num char = "; Cstr(Num_char)
	'If Success then
	'debug.print "Success"
	'Call PutPin(Int_LED,On) 'Turn off interrupt LED
	 'Call Togglepin(INT_LED)
	'Else
	'Debug.print "failure"
	'End if
    'Call UnLockTask()
    'Call Delay(0.015)
    'Call PutPin(Multi_Z_Slot_1_INT0,ON)  'High level for INTO for Multi-Z CPU in slot 1 / J1 	
	Loop
End Sub
'===================================================================================
'                          End of CPU 1 Interrupt 0 TaskStack1  
'===================================================================================


'===================================================================================
'                          From CPU 3 Interrupt 1 TaskStack2  
'===================================================================================
Sub CPU_3_INT1() 'From CPU 3 in slot 3 / J3 on INT1
'dim x as byte
	'Do
	'Warning - Do not use this routine if the cabling between Multi-Z CPUs is not connected!
	'Call WaitForInterrupt(zxPinLow,1) 'Wait for low level from Multi-Z CPU in slot J3
	'Wait 10-15 ms for data in buffer (10 bytes)

    'Call PutPin(Int_LED,On)  'Turn on interrupt LED to show interrupt from comm. Multi-Z CPU	

    'Call PutPin(Int_LED,OFF) 'Turn off interrupt LED
    'Call delay(1.0)	
	'Loop
End Sub
'===================================================================================
'                          End of CPU 3 Interrupt 0 TaskStack2  
'===================================================================================




'===================================================================================
' End of all code/program(s)
'===================================================================================






