'========================================================================================
'                 <Program Beginning For "Multi-Z CPU" in Slot 3 / J3>
'========================================================================================

'========================================================================================
'Module Title: Multi-Z CPU 3 R1.Bas - "Main Subroutine"
'========================================================================================
'Author: Zbasicandy
'========================================================================================
'Date: 7/1/06
'========================================================================================
'Operating System: XP
'========================================================================================
'ZBasic IDE Version: 1.1.1 / ZBasic Compiler version 1.29 / ZX-40 Firmware 1.2 / ZLoad v1.1.4
'========================================================================================
'Target: ZX-40 with 256k memory 
'========================================================================================
'Hardware: "Multi-Z CPU with BackPlane" 
'Support (Driver) Modules Needed:
'
'           ComSupport_R4.bas - for serial commuications - very useful communication routines 
'           Z40_Pin_I_O_R2.bas - for generic / ZX-40 Pin Input/Output functions/routines          
'
'========================================================================================
'Comports used: Software Comports 3,4 on ZX-40x for "Multi-Z CPU" hardware. 
'========================================================================================
'Bugs fixes and updates:
'
'
'
'
'
'
'****Warning**** This code is subject to change due to enhancements and bug fixes. 
'Treat this code as prototype code!
'
'
Option Explicit            'default (OFF) - implicit variable definition not used
Option Language Zbasic     'default
Option Base 1              'arrays start with 1 not 0
Option TargetCPU ZX40      'ZX40 (ZX-40 Atmega32); ZX40a (ZX-40a ATmega644)
Option CodeLimit 32K       '(1024 blocks)error if exceeded
Option SignOn On           'show sign-on on reset or boot
Option StringSize 40       '40 chars (default is 20) 'matches buffers
Option Strict Off          'zbasic default
Option AllocStr On         'zbasic default
Option Strict Off          'default
'
'============================================================================
'Pin State Meaning:
'zxInputTriState The pin should be a tri-state input (pull-up disabled).
'zxInputPullUp   The pin should be an input with the pull-up enabled.
'zxOutputLow     The pin should be an output set to logic zero.
'zxOutputHigh    The pin should be an output set to logic one.
'Warning - Keep all comports Tx(s) high and Rx(s) pulled high in software (idle)
'This prevents char ejects on start-up
'============================================================================
'                         I/O options - For ZX-40x ONLY!
'============================================================================
'Note: All pins are initilize as an input on power up.
Option Pin1  zxOutputLow        'PB0,  J2 pin 22, JR2 pin 1
Option Pin2  zxOutputLow        'PB1,  J2 pin 10, JL2 pin 2
Option Pin3  zxOutputLow        'INT2, J2 pin 23, JL2 pin 1
Option Pin4  zxOutputLow        'PB3,  J2 pin 11, JR1 pin 2
'-------------------------------------------------------------------------
'Do not change or enable!
'For reference only
'------------------------
'SPI / EEProm
'Option Pin5  zxInputtristate   'SS CS for SPI - for ref only
'Option Pin6  zxInputtristate   'MOSI SPI - for ref only
'Option Pin7  zxInputtristate   'MISO SPI - for ref only
'Option Pin8  zxInputtristate   'SCK SPI - for ref only
'Com1 program port
'Option Pin14 zxInputPullup     'Com1 Rx - Download port - for ref only
'Option Pin15 zxOutputHigh      'Com1 Tx - Download port - for ref only
'-------------------------------------------------------------------------

Option Pin16 zxInputTristate    'INT0, J3 pin 3 (Multi-processing connector J3) Do not change
Option Pin17 zxInputPullUp      'INT1, J3 pin 4 (Multi-processing connector J3)
Option Pin18 zxOutputLow        'PD4,  Run LED -> High = On Low = Off
Option Pin19 zxOutputHigh       'PD5,  J3 pin 1 (Multi-processing connector J3) Pulled-up on board
Option Pin20 zxOutputHigh       'PD6,  J3 pin 2 (Multi-processing connector J3) Pulled-up on board

Option Pin21 zxOutputLow        'PD7,      J2 pin 9,  JR2 pin 2 
Option Pin22 zxOutputLow        'SCL I2C,  J2 pin 21, JL3 pin 1
Option Pin23 zxOutputLow        'SDA I2C,  J2 pin  8, JL3 pin 2 
Option Pin24 zxInputPullUp      'Rx3,Com3  J2 pin  7, JR3 pin 2 <--RS232-converted on CPU x Board
Option Pin25 zxOutputHigh       'Tx3,Com3  J2 pin 20, JR3 pin 1 <--RS232-converted on CPU x Board
Option Pin26 zxOutputLow        'PC4,      J2 pin 19, JL4 pin 1
Option Pin27 zxOutputLow        'PC5,      J2 pin  6, JL4 pin 2
Option Pin28 zxOutputLow        'PC6,      J2 pin 18, JR4 pin 1
Option Pin29 zxOutputLow        'PC7,      J2 pin  5, JR4 pin 2

Option Pin33 zxOutputLow        'PA7,      J2 pin 17, JL5 pin 1 
Option Pin34 zxOutputLow        'PA6,      J2 pin  6, JL5 pin 2
Option Pin35 zxOutputLow        'PA5,      J2 pin 16, JR5 pin 1
Option Pin36 zxOutputLow        'PA4,      J2 pin  3, JR5 pin 2
Option Pin37 zxOutputLow        'PA3,      J2 pin 15, JL6 pin 1
Option Pin38 zxOutputLow        'PA2,      J2 pin  2, JL6 pin 2
Option Pin39 zxOutputLow        'PA1,      J2 pin 14, JR6 pin 1   
Option Pin40 zxOutputLow        'PA0,      Interrupt LED - On CPU Board - High = On & Low = Off
'========================================================================================
'                            End of I/O Options for ZX-40x
'========================================================================================
'------------------------
'Multi-Z Multi-Processing 
'------------------------

'Description/Operation

'Scheme #1 Master/Slave(s), TTL outputs & serial TTL buffer Rx4/RX5 inputs / INT0/INT1 interrupts
'Backplane CPU  ZX40x in slot  2 - J2 acts at the Master.
'Backplane CPUs ZX40x in slots 1 - J1 or slot 3 - J3 acts as the Slaves.
'The Master will initiate/poll all requests to the Slaves for data/variables.
'The cabling/wiring between the ZX-40x microcontrollers on J3 dictate the Multi-processing
'configuration.(NOT the CPU board slot/position in the rack/backplane) See below or schematic.
'This means you really don't need the backplane except for I/O and power distribution. 
'To extend multi-tasking a long distance between cards use a 4-20 ma interface circuits on all 
'of the TTL Inputs/Outputs on J3.

'Multi-Tasking CPU connector J3 has 4 TTL Inputs/Outputs. -> 2 TTL Outputs (Pulled up) & 
'2 TTL Inputs. The two outputs can be Tx4 and Tx5 serial buffers outputs. You can use them as 
'general purpose TTL outputs by using the PutPin(pin,state) command. The two inputs can be 
'general input interrupts INT0, INT1 or two Rx4 / Rx5 TTL serial inputs which have buffers/queues.
'
'The Master will poll a Slave by changing one of the TTL Tx outputs from a High/1/True to
'a Low/0/False condition. This output will feed the Slave TTL input INT0. The waitForInterrupt
'command in a Task will see the Low Level and trigger the interrupt service routine. In the
'interrupt service routine, a call is made to send TTL serial user data from the Slave TTL Tx
'output buffer to the Rx serial input buffer of the communications CPU Master.
'
'
'-------------------------------------------------------
' Multi-Z CPU Multi-Processing I/O Selection Table Setup 
' For reference only.
'------------------------------------------------------------------------
' Card / Slot Position                       |  J1  |  J2 Comm   | J3    |
'     CPU Selection -->                      |  X   |            |       |
'                                            |Slave1|  Master    |Slave2 |
'------------------------------------------------------------------------
' Multi-Proc. Con. J3 Pin 1  ZX-40x pin 19   | Tx4  | Out / Tx4  | Tx4   |
'                  J3 Pin 2             20   | Out  | Out / Tx5  | Out   |
'                  J3 Pin 3             16   | INT0 | Rx4 / INT0 | INT0  |
'                  J3 Pin 4             17   | Rx4  | Rx5 / INT1 | Rx4   |
'------------------------------------------------------------------------
' J3 Cable Wiring for Multi-Processing
'---------------------------------------
'Note: Pin 1 is the closest to the top edge of the CPU board.
'Note: Quantity of 6 1K series current limiting resistor(s) are inserted between all 
'wiring connections for protection. The resistors are soldered and heat shrink in the cabling.
'Note: Check schematic for the Multi-Z J3 wiring cabling diagram between Multi-Z CPUs.
'Pin 1 of J1 goes to Pin 3 of J2 - for Multi-Z CPU slot 1 or J1
'Pin 3 of J1 goes to Pin 1 of J2 - for Multi-Z CPU slot 1 or J1
'Pin 1 of J3 goes to Pin 4 of J2 - for Multi-Z CPU slot 3 or J3
'Pin 3 of J3 goes to Pin 2 of J2 - for Multi-Z CPU slot 3 or J3
'--------------------------------------------------------------

'=========================================================================================
'Program Definitions:
'=========================================================================================
'
'
'============================================================================
'Constants:
'[Public | Private] Const <name> As <type> = <value> (default private)
'============================================================================

'-----------------------------
' Port Constants for all ports
'-----------------------------
' Do not modify!
Private Const Port3 as byte  = 3           
Private Const Port4 as byte  = 4
'Private Const Port5 as byte = 5
'Private Const Port6 as byte = 6
'-----------------------------

' General constants
Private Const Run_LED as byte = 18
Private Const Int_LED as byte = 40
Private Const INT0    as byte = 16
Private Const On      as byte =  1
Private Const Off     as byte =  0

'============================================================================
'Variables:
'{Public | Private | Dim} <name> As <type> (default private)
'============================================================================
Private CPU_1_Comm_Data(1 to 10) as byte ' Array (1 Base) for Multi-Z CPU communications
Private Global_Slave_Enable as boolean 'Enable / Disable Slave "waitforinterrupt task"

'============================================================================
'Subroutine declarations:
'============================================================================
Public Sub Send_CPU_1_Data() 'Send status/variable data to Multi-Z communication CPU
Dim i as integer
Dim Tx_Str as string

Call Compute_Array_Checksum()
    
'Send By Bytes OR
	'For i = LBound(CPU_1_Comm_Data) to UBound(CPU_1_Comm_Data)	
	'Call PutByte(Port4,CPU_1_Comm_Data(i))
    'Debug.Print "Transmit Character = "; cstr(CPU_1_Comm_Data(i)) ; " Index = "; cstr(i)
	'Call delay(0.0001)
	'Next

'Send By String - Have one only - comment the other one out!
	Tx_Str = ""
	For i = LBound(CPU_1_Comm_Data) to UBound(CPU_1_Comm_Data)	
	Tx_Str = Tx_Str & Chr(CPU_1_Comm_Data(i)) 
	Next
	'Debug.print "Tx string "; Tx_Str
	Call PutStr(Port4, Tx_Str)
End Sub
'----------------------------------------------------------------------------
Private Sub Com3_Turn_Around_Test() 
'Install jumper on RS232 connector on pins 2-3 on the backplane
Dim OK as Boolean 'For com3 turnaround test
static Success_Counter as Long
static Failure_Counter as Long
Call Comport_TurnaroundTest(Port3, OK )
    If OK then
		'Debug.Print "Port 3 is OK"
		Success_Counter = Success_Counter + 1
		'Debug.Print "Port 3 Success_Counter = "; Cstr(Success_Counter)
	Else
		'Debug.Print "Port 3 is Not OK!"
		Failure_Counter = Failure_Counter + 1		
		Debug.Print "Port 3 Failure_Counter = "; Cstr(Failure_Counter)
		Call ClearOutputBuffer(Port3)
		Call ClearInputBuffer(Port3)
End If
End Sub
'-----------------------------------------------------------------------------
Private Sub Com4_Turn_Around_Test() 'Local on board TTL Serial Turnaround Test 
'Install jumper with a series 1k resistor on J3 pins 1-4 on the Multi-Z CPU 
Dim OK as Boolean 'For com3 turnaround test
static Success_Counter as Long
static Failure_Counter as Long
Call Comport_TurnaroundTest(Port4, OK )
	If OK Then
		'Debug.Print "Port 4 is OK"
		Success_Counter = Success_Counter + 1
		'Debug.Print "Port 4 Success_Counter = "; Cstr(Success_Counter)
	Else
		'Debug.Print "Port 4 is Not OK!"
		Failure_Counter = Failure_Counter + 1		
	    Debug.Print "Port 4 Failure_Counter = "; Cstr(Failure_Counter)
		Call ClearOutputBuffer(Port4)
		Call ClearInputBuffer(Port4)
End if
End Sub
'-------------------------------------------------------------------------------------------
Private Sub Cycle_LEDS() 'Cycle all LEDs on Power-up or Reset - For troubleshooting
Dim i as Integer
For i = 0 to 5 ' cycle 6 times
Call PutPin(Run_LED,On)
Call PutPin(Int_LED,On)
Call Delay(0.2)
Call PutPin(Run_LED,Off)
Call PutPin(Int_LED,Off)
Call Delay(0.2)
Next
End Sub
'----------------------------------------------------------------------------
Private Sub Compute_Array_Checksum()
Dim i as integer
Dim CS as byte
CS = 0
For i = LBound(CPU_1_Comm_Data) to ((UBound(CPU_1_Comm_Data)) - 1)	
CS = CS + CPU_1_Comm_Data(i) ' Sum all bytes 
Next
CS = Not CS  ' Ones Complement
CS = CS + 1  ' Add 1
'Debug.print "Computed Checksum = "; CstrHex(CS)
'Debug.print "Array Max = "; Cstr((UBound(CPU_1_Comm_Data)) - 1)
CPU_1_Comm_Data(UBound(CPU_1_Comm_Data)) = CS 'store checksum in last byte of array 
End Sub
'----------------------------------------------------------------------------
Private Sub BackPlane_IO_Test()
'Cycle all I/O for Backplane Test
'DO NOT PLUG ANY TTL LOGIC INTO JR3 PINS 1 & 2 E.G. RS232 Tx & Rx
Call PutPin(1,ON)
Call PutPin(2,ON)
Call PutPin(3,ON)
Call PutPin(4,ON)

'Call PutPin(6,ON) <--SPI Buss Do Not Use
'Call PutPin(7,ON) <--SPI Buss Do Not Use
'Call PutPin(8,ON) <--SPI Buss Do Not Use

Call PutPin(21,ON)
Call PutPin(22,ON)
Call PutPin(23,ON)
'Call PutPin(24,ON) <--RS232 Do Not Use
'Call PutPin(25,ON) <--RS232 Do Not Use
Call PutPin(26,ON)
Call PutPin(27,ON)
Call PutPin(28,ON)
Call PutPin(29,ON)
Call PutPin(33,ON)
Call PutPin(34,ON)
Call PutPin(35,ON)
Call PutPin(36,ON)
Call PutPin(37,ON)
Call PutPin(38,ON)
Call PutPin(39,ON)

Call Delay(1.0)

Call PutPin(1,OFF)
Call PutPin(2,OFF)
Call PutPin(3,OFF)
Call PutPin(4,OFF)

'Call PutPin(6,ON)  <--SPI Buss Do Not Use
'Call PutPin(7,ON)  <--SPI Buss Do Not Use
'Call PutPin(8,ON)  <--SPI Buss Do Not Use

Call PutPin(21,OFF)
Call PutPin(22,OFF)
Call PutPin(23,OFF)
'Call PutPin(24,OFF) <--RS232 Do Not Use
'Call PutPin(25,OFF) <--RS232 Do Not Use
Call PutPin(26,OFF)
Call PutPin(27,OFF)
Call PutPin(28,OFF)
Call PutPin(29,OFF)
Call PutPin(33,OFF)
Call PutPin(34,OFF)
Call PutPin(35,OFF)
Call PutPin(36,OFF)
Call PutPin(37,OFF)
Call PutPin(38,OFF)
Call PutPin(39,OFF)

Call Delay(1.0)

End Sub
'============================================================================	 
'
'
'============================================================================
'Function declarations:
'============================================================================



'============================================================================
'Tasks Init. Setup
'============================================================================
Dim TaskStack1(1 to 70) as Byte  '
'Dim TaskStack2(1 to 65) as Byte ' spare
'Dim TaskStack3(1 to 50) as Byte ' spare
'============================================================================

Sub Main()
Dim OK as boolean 'For com3 turnaround test
Dim Status as boolean

'------------------------------------------------------------------------------------------
'                       Serial Port(s) Communications Setup
'------------------------------------------------------------------------------------------

'----------------------------------------------------
'Set number of com channels and max baud rate
'----------------------------------------------------
Call ComChannels(2, 9600) ' Max 9600 using 4 comports
'----------------------------------------------------
'Define "Multi-Z CPU 1" ZX-40x Commuications Serial I/O
'---------------------------------------------------

Call DefineCom(3, 24, 25, &H08) ' Com 3 ,rec pin 24 tran pin 25 - 8,N,1 <--- External RS232 communications port 
Call DefineCom(4, 17, 19, &H08) ' Com 4 ,rec pin 17 tran pin 19 - 8,N,1 <--- For Multi-Processing 
'Call DefineCom(5, x, x, &H08) ' Com 5 ,rec pin  x tran pin x - 8,N,1  
'Call DefineCom(6, x, x, &H08) ' Com 6 ,rec pin  x tran pin x - 8,N,1 

'------------------------------
'Software Serial ports "opened"
'------------------------------
Call OpenSerialPort(3,9600) 'Com 3 <--RS232-converted on CPU Board (JR3 pins 1 & 2 - Tx & Rx) 
Call OpenSerialPort(4,9600) 'Com 4 <-- For TTL Multi-Z CPU Processing - Do Not Use. 
'Call OpenSerialPort(5,9600) 'Com 5 
'Call OpenSerialPort(6,9600) 'Com 6 

'-----------------------------------------------------------------------------------------
'                          End Of Communications Setup
'-----------------------------------------------------------------------------------------
Debug.Print "<Slave Controller Boot/Reset>"
Call ClearOutputBuffer(Port4) 'Clear TX4 buffer before 
Call Cycle_LEDs() 'For troubleshooting
Global_Slave_Enable = True 'Enable Slave Waitforinterrupt INT0 Task

'===============================
'Call Task(s) for multi-tasking.
'===============================
If Global_Slave_Enable Then
CallTask "CPU_1_3_INT0", TaskStack1  'Check for Multi-Tasking - make sure cabling is installed!
End If
'CallTask " ", TaskStack2    'spare
'CallTask " ", TaskStack3    'spare
'-------------------------------

'---------------------------
'CPU x System data/variables
'---------------------------
CPU_1_Comm_Data(1) =  48 '0
CPU_1_Comm_Data(2) =  49 '1
CPU_1_Comm_Data(3) =  50 '2
CPU_1_Comm_Data(4) =  51 '3
CPU_1_Comm_Data(5) =  52 '4
CPU_1_Comm_Data(6) =  53 '5
CPU_1_Comm_Data(7) =  54 '6
CPU_1_Comm_Data(8) =  55 '7
CPU_1_Comm_Data(9) =  56 '8
CPU_1_Comm_Data(10) = 00 '   <Checksum = , e.g. &H2c> <--- Do not use - last byte is computed 
'                                 checksum of 9 previous bytes

Call Compute_Array_Checksum()

Debug.Print "Array Checksum = "; cstrhex(CPU_1_Comm_Data(UBound(CPU_1_Comm_Data)))

Do
Call BackPlane_IO_Test()

CALL TogglePin(Run_LED) 'Toggle the red RUN LED - Multi-Z CPU Running indication.
Loop

End Sub










'===================================================================================
'                         End of "Main" Application Subroutine
'===================================================================================




'===================================================================================
'                          CPU 1 Interrupt 0 TaskStack1 - Scheme #1 
'===================================================================================
Sub CPU_1_3_INT0()
Dim x as byte
	Do
	'Warning - Do not use this routine if the cabling between Multi-Z CPUs is not connected!
	Call WaitForInterrupt(zxPinLow,0) 'Wait for low level from Multi-Z Com CPU in slot 2 / J2
	'Low level detected
	Call Locktask() ' Prevent other tasks from running until finished	
    Call PutPin(Int_LED,On)  'Turn on interrupt LED to show interrupt from Comm. Multi-Z CPU slot 2 / J2
	x = Getpin(INT0) 'check for high level
	
	If x = 0 then
		Call Delay(0.001) ' 1 ms timeout delay
		x = Getpin(INT0) 'check for high
		If x = 1 then 'recheck for high with 1 ms timeout delay
			Goto Send_Data
		Else
		    Call PutPin(Int_LED,OFF) 'Turn off interrupt LED status indicator
			Debug.Print "Exiting Task"
		    ExitTask(TaskStack1)
		End If
	End If
    'Debug.Print "INT0 Pin = "; Cstr(x)
	Send_Data:
    Call ClearOutputBuffer(Port4) 'Clear TX4 buffer before sending data	
    Call Send_CPU_1_Data() ' 10 bytes including checksum
    Call PutPin(Int_LED,OFF) 'Turn off interrupt LED status indicator
	Call Delay(0.010) 'For ten bytes
	Call UnLockTask() 
	Loop
End Sub
'===================================================================================
'                          End of CPU 1 Interrupt 0 TaskStack1  
'===================================================================================





'===================================================================================
' End of all code/program(s)
'===================================================================================


